# MouseOps.py
# Name: Alex Cappiello
# AndrewID: acappiel
# Section: H
# Term project for 15-110, spring 2011
# All code original work unless otherwise noted

import Gui, DrawingTools
from PIL import Image, ImageDraw

#classes control the various mouse functions
#delta compensates for Tkinter oddities
#only draw with PIL when necessary

class Brush:
    
    def leftClick( self, x, y ):
        delta = canvas.data.drawOffset
        r = canvas.data.lineWidth / 2
        canvas.create_oval( x-r, y-r, x+r, y+r,
                            fill = canvas.data.activeTool[ 1 ], width = 0 )
        canvas.data.draw.ellipse( [ ( x - r - delta, y - r - delta),
                                    ( x + r - delta, y + r - delta) ],
                                  fill = canvas.data.activeTool[ 2 ] )
    
    def drag( self, x, y ):
        delta = canvas.data.drawOffset
        canvas.create_line( canvas.data.x, canvas.data.y, x, y,
                            fill = canvas.data.activeTool[ 1 ],
                            width = canvas.data.lineWidth + 1 )
        canvas.data.draw.line( [ ( canvas.data.x - delta, canvas.data.y -delta),
                                 ( x - delta, y - delta ) ],
                               fill = canvas.data.activeTool[ 2 ],
                               width = canvas.data.lineWidth)
        canvas.data.x = x
        canvas.data.y = y

    def release( self, x, y ):
        self.leftClick( x, y )

    def rightClick( self, x, y ): pass

class Line:
    
    def leftClick( self, x, y ):
        canvas.data.newShape = canvas.data.activeTool[ 1 ]( x, y, x, y,
                                                 fill = canvas.data.hexColorA,
                                                 width = canvas.data.lineWidth )

    def drag( self, x, y ):
        canvas.delete( canvas.data.newShape )
        canvas.data.newShape = canvas.data.activeTool[ 1 ]( canvas.data.x,
                                                 canvas.data.y, x, y,
                                                 fill = canvas.data.hexColorA,
                                                 width = canvas.data.lineWidth )
    def release( self, x, y ):
        delta = canvas.data.drawOffset
        canvas.data.activeTool[ 2 ]( [ ( canvas.data.x - delta,
                                         canvas.data.y - delta ),
                                       ( x - delta, y - delta )],
                                     fill = canvas.data.rgbColorA,
                                     width = canvas.data.lineWidth)

    def rightClick( self, x, y ): pass

class DrawShape:

    def leftClick( self, x, y ):
        canvas.data.newShape = canvas.data.activeTool[ 1 ]( x, y, x, y,
                                               fill = canvas.data.hexColorB,
                                               width = canvas.data.lineWidth,
                                               outline = canvas.data.hexColorA )

    def drag( self, x, y ):
        canvas.delete( canvas.data.newShape )
        canvas.data.newShape = canvas.data.activeTool[ 1 ]( canvas.data.x,
                                               canvas.data.y, x, y,
                                               fill = canvas.data.hexColorB,
                                               width = canvas.data.lineWidth,
                                               outline = canvas.data.hexColorA )

    def release(self,x,y):
        delta = canvas.data.drawOffset
        border = canvas.data.lineWidth / 2
        corners = findCorners( canvas.data.x, canvas.data.y, x, y)
        #PIL does not have a border width option, so draw it as a nested shape
        canvas.data.activeTool[ 2 ]( [ ( corners[ 0 ][ 0 ] - border - delta,
                                         corners[ 0 ][ 1 ] - border - delta),
                                       ( corners[ 1 ][ 0 ] + border - delta,
                                         corners[ 1 ][ 1 ] + border - delta ) ],
                                     fill = canvas.data.rgbColorA )
        canvas.data.activeTool[ 2 ]( [ ( corners[ 0 ][ 0 ] + border - delta,
                                         corners[ 0 ][ 1 ] + border - delta ),
                                       ( corners[ 1 ][ 0 ] - border - delta,
                                         corners[ 1 ][ 1 ] - border - delta ) ],
                                     fill = canvas.data.rgbColorB )

    def rightClick( self, x, y ): pass

def findCorners( x0, y0, x1, y1 ):
    topLeftX = min( x0, x1 )
    topLeftY = min( y0, y1 )
    bottomRightX = max( x0, x1 )
    bottomRightY = max( y0, y1 )
    beginning = ( topLeftX, topLeftY )
    end = ( bottomRightX, bottomRightY )
    return [ beginning, end ]

class Fill:

    def endCase( self, x, y, pix, baseColor ):
        c1 = x < 0 or y < 0
        c2 = x >= canvas.data.size[ 0 ] or y >= canvas.data.size[ 1 ]
        if ( not c1 and not c2 ):
            c3 = pix[ x, y ] != baseColor
        return c1 or c2 or c3

    def floodFill( self, x, y, pix, baseColor ):
        '''
        pix[ x, y ] = canvas.data.rgbColorA
        new = [ ( x - 1, y ), ( x + 1 , y ), ( x, y - 1),( x, y + 1 ) ]
        elt = 0
        x = True
        while( x ):
            if ( self.endCase( new[ elt ][ 0 ], new[ elt ][ 1 ], pix,
                               baseColor ) ):
                new.pop( elt )
            else:
                elt += 1
            x = elt < len( new )
        return new
        '''
        #8 seconds
        if ( self.endCase( x, y, pix, baseColor ) ):
            # base case, do nothing!
            return []
        else:
            pix[ x, y ] = canvas.data.rgbColorA
            new = [ ( x - 1, y ), ( x + 1 , y ), ( x, y - 1),( x, y + 1 ) ]
            return new

    def leftClick( self, x, y ):
        depth = 0
        pix = canvas.data.rawImage.load()
        delta = canvas.data.drawOffset
        x -= delta; y -= delta
        baseColor = pix[ x, y ]
        Q = [ ( x, y ) ]
        while ( len( Q ) > 0 ):
            x = Q[ 0 ][ 0 ]
            y = Q[ 0 ][ 1 ]
            Q += self.floodFill( x, y, pix, baseColor )
            Q.pop( 0 )

    def rightClick( self, x, y ): pass
    def drag( self, x, y ): pass
    def release( self, x, y): pass

class Eyedrop:

    def leftClick( self, x, y ):
        rgbColor = canvas.data.rawImage.load()[ x, y ]
        hexColor = ( "#%02x%02x%02x" % rgbColor )
        Gui.buttonColor = hexColor
        canvas.data.colorButtonA.config( bg=hexColor )
        DrawingTools.setColor( [ rgbColor, hexColor ], 1 )

    def rightClick( self, x, y ):
        rgbColor = canvas.data.rawImage.load()[ x, y ]
        hexColor = ( "#%02x%02x%02x" % rgbColor )
        Gui.buttonColorB = hexColor
        canvas.data.colorButtonB.config( bg=hexColor )
        DrawingTools.setColor( [ rgbColor, hexColor ], 2 )
        
    def drag( self, x, y ): pass
    def release( self, x, y ): pass

class Polygon:

    def __init__(self):
        self.vertexList = []

    def leftClick( self, x, y ):
        delta = canvas.data.drawOffset
        canvas.data.temp = canvas.create_line( x, y, x, y )
        self.vertexList.append( ( x - delta, y - delta ) )

    def rightClick( self, x, y ):
        if ( len( self.vertexList ) > 0 ):
            delta = canvas.data.drawOffset
            lastCoord = self.vertexList[ len( self.vertexList ) - 1 ] 
            canvas.create_line( lastCoord[ 0 ] + delta, lastCoord[ 1 ] + delta,
                                x, y, fill = canvas.data.hexColorA,
                               width = canvas.data.lineWidth)
            self.vertexList.append( ( x - delta, y - delta ) )

    def drag( self, x, y ):
        canvas.delete( canvas.data.temp )
        delta = canvas.data.drawOffset
        lastCoord = self.vertexList[ len( self.vertexList ) - 1 ]
        canvas.data.temp = canvas.create_line( lastCoord[ 0 ] + delta,
                                               lastCoord[ 1 ] + delta, x, y )

    def release( self, x, y ):
        if ( len( self.vertexList ) > 2 ):
            canvas.data.draw.polygon( self.vertexList,
                                      fill = canvas.data.rgbColorA )
        self.vertexList = []
